# Skidtools, A collection of utilities and tools to rapidly develop CLI programs. 
# It aims to get rid of boilerplate and aid skiddies.
# Copyright (C) 2020 Abdos

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import sys
from typing import List, Dict, Optional

try:
    from skidtools import config_template
except Exception:
    import config_template


def command_line_tools(sys_arg: List[str]) -> None:

    opts: List[str] = [opt.lower() for opt in sys_arg[1:] if opt.startswith("--")]
    flags: List[str] = [flag.lower() for flag in sys_arg[1:] if flag.startswith("-")]

    # Options
    for opt in opts:
        if opt == "--help":
            print("--init: Initilize config.py")
            return

        elif opt == "--init":
            with open(config_template.__file__, "r") as s:
                template: str = s.read()
                with open("config.py", "w") as f:
                    f.write(template)
            return
            
        else:
            raise SystemExit(f"Invalid option: {opt}, see --help for usage.")

    for flag in flags:
        if flag not in []:
            raise SystemExit(f"Invalid flag: {flag}, see --help for usage.")

if __name__ == "__main__":
    command_line_tools(sys.argv)



# Default config for skidtool
# See documumention for details docs.clast.dev/skidtools/home/config/
# https://github.com/Abd0s/skidtools/

from typing import List, Dict, Union, Optional

# General
version: str = "0.0.1"
help_text: str = "A simple help accesed through --help"

# slogging
log_to_file: bool = False
debug: bool = False


# Skidtools, A collection of utilities and tools to rapidly develop CLI programs. 
# It aims to get rid of boilerplate and aid skiddies.
# Copyright (C) 2020 Abdos

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

""" Libary specific exceptions

These exceptions are raised when no appropriate builtin exception exists.
"""
class MissingConfigFile(Exception):
    """Raised when no config.py file is found"""
    pass


# Skidtools, A collection of utilities and tools to rapidly develop CLI programs. 
# It aims to get rid of boilerplate and aid skiddies.
# Copyright (C) 2020 Abdos

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

""" Wrapper around the standard logging module.

Using the logging module can be very beneficial, but it often requres alot of setup code to get the desired result.
This submodule wraps the standard logging module with a defualt configuration while also adding CLI options and flags with very little code.
A config.py file is required for this module.

Typical usage example:
```python
from skidtools import slogging
import logging

init_logging()
logging.error("An error happend!")
```
"""

from typing import Dict, List, Union, Optional
import logging
from datetime import datetime
import pathlib
import sys

from .exceptions import MissingConfigFile
from . import sio


# a config.py file is required for this mnodule
try:
    import config
except ModuleNotFoundError:
    raise MissingConfigFile("No config.py file found, use --init to initilize one.")


def init_logging(sys_arg: List[str] = sys.argv) -> None:
    """Initializes the logging module

    Initializes the root logger of the logging module with support for the following CLI arguments (overwrites config.py):

    Options:
        --version: prints the version provided in config.py
        --help: prints the help string provided in config.py
    Flags:
        -d: Sets the logging level to debug
        -f: Write the logging output to a logfile instead of sysout.

    Args:
        sys_arg: A list of system CLI arguments, defaults to sys.argv.

    Returns:
        None

    Raises:
        SystemExit: An Invalid option or flag is passed.
    """
    # Command line options and flags
    opts: List[str] = [opt.lower() for opt in sys_arg[1:] if opt.startswith("--")]
    flags: List[str] = [flag.lower() for flag in sys_arg[1:] if flag.startswith("-")]
    
    # Options
    for opt in opts:
        if opt == "--version":
            print(f"version: {config.version}")
            return

        elif opt == "--help":
            print(config.help_text)
            return

        else:
            raise SystemExit(f"Invalid option: {opt}, see --help for usage.")

    # Flags
    if "-f" in flags:
        config.log_to_file: bool = True
    if "-d" in flags:
        config.debug: bool = True

    for flag in flags:
        if flag not in ["-f", "-d"]:
            raise SystemExit(f"Invalid flag: {flag}, see --help for usage.")

    # Logging
    log_filename: str = "D" + datetime.now().isoformat('T', 'seconds').replace(':', '-')

    if config.debug:
        logging.root.setLevel(logging.DEBUG)
    else:
        logging.root.setLevel(logging.INFO)

    if config.log_to_file:
        # Make dir if it doesn't exist
        pathlib.Path("./logs").mkdir(parents=True, exist_ok=True)

        # define file handler and configure
        file_handler: logging.FileHandler = logging.FileHandler(filename=f"logs/{log_filename}.log", mode="w")
        formatter: logging.Formatter = logging.Formatter("%(asctime)s %(name)-12s %(levelname)-8s %(funcName)-12s %(lineno)-4s %(message)s", datefmt='%Y-%m-%d %H:%M:%S')
        file_handler.setFormatter(formatter)

        # add the handler to the root logger
        logging.root.addHandler(file_handler)
        
    # define console handler and configure
    console_handler: logging.StreamHandler = logging.StreamHandler()
    console_handler.setFormatter(ConsoleFormatter())

    # add the handler to the root logger
    logging.root.addHandler(console_handler)

class ConsoleFormatter(logging.Formatter):
    """Logging Formatter to add colors and formatting"""

    FORMATS = {
        logging.CRITICAL: f"[{sio.colored_print('CRITICAL', 'yellow')}] %(name)s: %(message)s",
        logging.ERROR: f"[{sio.colored_print('ERROR', 'red')}] %(name)s: %(message)s",
        logging.WARNING: f"[{sio.colored_print('WARNING', 'pink')}] %(name)s: %(message)s",
        logging.INFO : f"[{sio.colored_print('INFO', 'blue')}] %(name)s: %(message)s",
        logging.DEBUG: f"[{sio.colored_print('DEBUG', 'cyan')}] %(name)s: %(message)s",
        "DEFAULT": "[LOG]: %(name)s: %(msg)s",
    }

    def format(self, record: logging.LogRecord) -> str:
        log_fmt = self.FORMATS.get(record.levelno, self.FORMATS['DEFAULT'])
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


# Skidtools, A collection of utilities and tools to rapidly develop CLI programs. 
# It aims to get rid of boilerplate and aid skiddies.
# Copyright (C) 2020 Abdos

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

# Load list of dependencies
with open("requirements.txt") as data:
    install_requires = [
        line for line in data.read().split("\n")
            if line and not line.startswith("#")
    ]

setuptools.setup(
    name="skidtools", # Replace with your own username
    version="0.0.1a3",
    author="Abdos",
    author_email="admin.dev@clast.dev",
    description="A small utility package to reduce boilerplate code in skid scripts",
    long_description= long_description,
    long_description_content_type="text/markdown",
    url="https://docs.clast.dev/skidtools/",
    packages = setuptools.find_packages(exclude = ["tests"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Topic :: Utilities",
        "Typing :: Typed"
    ],
    python_requires='>=3.6',
    install_requires = install_requires
)


# Skidtools, A collection of utilities and tools to rapidly develop CLI programs. 
# It aims to get rid of boilerplate and aid skiddies.
# Copyright (C) 2020 Abdos

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

""" A collection of functions to handle commen IO task

Contains a collection of utility functions designed to speed up commen IO cases such as loading combos or printing text with color

Typical usage example:
```python
from skidtools import sio

combos = load_combos()
# Do stuff with the combos
```
"""

from typing import Any, List, Dict, Union
from colorama import init, Fore, Style
init(autoreset=True)
from tqdm import tqdm # type: ignore

def colored_print(text: str, color: str) -> str:
    """ Changes color of a string when printed

    Adds ANSI escape character sequences (for producing colored terminal prints) to the given string.

    Args:
        text: Text to color.
        color: Color to apply to the given text. List of available colors: green, red, blue, cyan, yellow, pink.

    Returns:
        A string which contains the original text wrapped in ANSI escape characters.
    """

    if color.upper() == "GREEN":
        return f"{Fore.GREEN}{text}{Style.RESET_ALL}"
    elif color.upper() == "RED":
        return f"{Fore.RED}{text}{Style.RESET_ALL}"
    elif color.upper() == "BLUE":
        return f"{Fore.BLUE}{text}{Style.RESET_ALL}"
    elif color.upper() == "CYAN":
        return f"{Fore.CYAN}{text}{Style.RESET_ALL}"
    elif color.upper() == "YELLOW":
        return f"{Fore.YELLOW}{text}{Style.RESET_ALL}"
    elif color.upper() == "PINK":
        return f"{Fore.MAGENTA}{text}{Style.RESET_ALL}"
    else:
        raise ValueError("Invalid argument: color")

def validated_input(text: str, _type: Union[int, str, float]) -> Union[int, str, float]:
    """ Extension of the builtin input() function.

    Loops input() until it's return value is convertable to the given _type. P
    rints an error message with the correct type if given input isn't convertable wrong type is entered.

    Args:
        text: The text that should be printed with the input
        _type: The type the input should be validated and converted to.

    Returns:
        The CLI input converted to the given _type.
    """
    convertable_types: List[str] = [int, str, float]
    if _type not in convertable_types:
        raise ValueError(f"Invalid conversions type, must be one of {[type_.__name__ for type_ in convertable_types]}")

    while True:
        try:
            converted_input: _type = _type(input(text))
            break
        except ValueError:
            print(f"{colored_print('[-]', 'RED')} Invalid input, must be of type {_type.__name__}")

    return converted_input

def load_combos(filename: str = "combos.txt", seperator: str = ":", visualizer: bool = True) -> List[Dict[str, str]]:
    """ Loads and parses a combo formatted textfile

    Loads a file which contains password:email formatted data seperated with a seperator ":" by default.

    Args:
        filename: Path of the file that should be loaded, defaults to combos.txt.
        seperator: Characters that seperate the email and password part of a combo, defaults to :.
        visualizer: Progresbar that shows the loading and parsing progres using the tqdm package.

    Returns:
        A List of dicts mapping with keys email and password. For example {"email": "example@email.com", "password": "password"}

    Raises:
        FileNotFoundError: Couldn't find a file with given filename.
    """
    with open(filename, "r") as f:
        lines: List[str] = f.read().splitlines()

    combos: List[Dict[str, str]] = []    
   
    if visualizer:
        lines = tqdm(lines, desc=f"{colored_print('[+]', 'blue')} Loading and parsing combos", ncols=100)

    for line in lines:
        combo: List[str] = line.split(seperator)
        combo_dict: Dict[str, str] = {"email": combo[0], "password": combo[1]}
        combos.append(combo_dict)
    
    return combos




# Skidtools, A collection of utilities and tools to rapidly develop CLI programs. 
# It aims to get rid of boilerplate and aid skiddies.
# Copyright (C) 2020 Abdos

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

"""Collection of generators used for generation credentials and other data

Typical usage example:
```python
from skidtools import sfactory
credentials = sfactory.generate_credentials("number", "bot")
login(credentials['username']) # made up example function
```

"""

from typing import Dict, List, Union, Optional
from faker import Faker # type: ignore

# Faker instance used for fake data
fake = Faker()


def generate_credentials(engine: str = "name", prefix: Optional[str] = None) -> Dict[str, str]:
    """Generates a pair of random credentials (firstname, lastname, username, email, password).

    Args:
        engine: This defines the pattern used to generate the credentials, number generates using only numbers, leter genrates using only letters,
                name generates a random realistic name using the Faker module, mix is a combination of numbers and letters. This doesn't effect password
                genration or email. The email is in the username@skid.dev format.
        prefix: Optional argument which prefixes the given string to the the generated username and email

    Returns:
        A dict with with they keys: firstname, lastname, username, email, password
    """
    credentials: Dict[str, str] = {}

    if engine == "name":
        # Real names and data with Faker
        credentials['firstname'] = fake.first_name()
        credentials['lastname'] = fake.first_name()
        credentials['username'] = fake.user_name()
        credentials['email'] = fake.free_email()

    elif engine == "number":
        # Combination of random integers
        credentials['firstname'] = str(fake.random_number(digits=10, fix_len=True)) #turn into string
        credentials['lastname'] = str(fake.random_number(digits=10, fix_len=True)) 
        credentials['username'] = str(fake.random_number(digits=10, fix_len=True))
        credentials['email'] = credentials['username'] + "@skid.dev"

    elif engine == "letter":
        # Combination of random letters
        credentials['firstname'] = ''.join(fake.random_letters(length=10)) # turn into string
        credentials['lastname'] = ''.join(fake.random_letters(length=10))
        credentials['username'] = ''.join(fake.random_letters(length=10))
        credentials['email'] = credentials['username'] + "@skid.dev"

    elif engine == "mix":
        # A combination of numbers and letters
        credentials['firstname'] = fake.password(length=10, special_chars=False) # fake.password returns a combination of random numbers and letters
        credentials['lastname'] = fake.password(length=12, special_chars=False)
        credentials['username'] = fake.password(length=12, special_chars=False)
        credentials['email'] = credentials['username'] + "@skid.dev"
         
    else:
        raise ValueError("Invalid argument: engine")
    
    credentials['password'] = fake.password(length=12, special_chars=False)
    # Add prefix to username and email
    if prefix != None:
        credentials['username'] = prefix + credentials['username']
        credentials['email'] = prefix + credentials['email']

    return credentials